#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#========================================================================
#
# xlogparse
#
# DESCRIPTION
#
# Parses Xlog.*.log format files and allows looking up data from it
#
# AUTHOR
#   Bryce W. Harrington <bryce@canonical.com>
#
# COPYRIGHT
#   Copyright (C) 2010-2012 Bryce W. Harrington
#   All Rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#========================================================================

from __future__ import absolute_import, print_function, unicode_literals

import re

class XorgLog(object):

    def __init__(self, logfile=None):
        self.devices  = [ ]
        self.modules  = [ ]
        self.errors   = [ ]
        self.warnings = [ ]
        self.info     = [ ]
        self.notimpl  = [ ]
        self.notices  = [ ]
        self.displays = { }
        self.xserver_version = None
        self.boot_time = None
        self.boot_logfile = None
        self.kernel_version = None
        self.video_driver = None
        self.xorg_conf_path = None
        self.logfile = logfile

        if logfile:
            self.parse(logfile)

    def add_device(self, device, devclass, devtype):
        device = {
            'name':         device,
            'class':        devclass,
            'type':         devtype
            }
        self.devices.append(device)

    def parse(self, filename):
        self.displays = {}
        display = {}
        display_name = "Unknown"
        in_file = open(filename, "r")
        gathering_module = False
        found_ddx = False
        module = None
        for line in in_file.readlines():
            #print("Line: %s" % (line))

            # TODO: PCI
            # TODO: extensions

            # Modules and Devices
            m = re.search(r'\(..\)', line)
            if m:
                if gathering_module and module is not None:
                    self.modules.append(module)
                gathering_module = False
                module = None

                m = re.search('\(II\) Loading.*modules\/drivers\/(.+)_drv\.so', line)
                if m:
                    found_ddx = True

                m = re.search(r'\(II\) Module (\w+):', line)
                if m:
                    module = {
                        'name':         m.group(1),
                        'vendor':       None,
                        'version':      None,
                        'class':        None,
                        'abi_name':     None,
                        'abi_version':  None,
                        'ddx':          found_ddx,
                        }
                    found_ddx = False
                    gathering_module = True

            if gathering_module:
                   m = re.search(r'vendor="(.*:?)"', line)
                   if m:
                       module['vendor'] = m.group(1)

                   m = re.search(r'module version = (.*)', line)
                   if m:
                       module['version'] = m.group(1)

                   m = re.search(r'class: (.*)', line)
                   if m:
                       module['class'] = m.group(1)

                   m = re.search(r'ABI class:\s+(.*:?), version\s+(.*:?)', line)
                   if m:
                       if m.group(1)[:5] == "X.Org":
                           module['abi_name'] = m.group(1)[6:]
                       else:
                           module['abi_name'] = m.group(1)
                       module['abi_version'] = m.group(2)
                   continue

            # General details
            m = re.search(r'Current Operating System: (.*)$', line)
            if m:
                uname = m.group(1)
                self.kernel_version = uname.split()[2]
                continue

            m = re.search(r'Kernel command line: (.*)$', line)
            if m:
                self.kernel_command_line = m.group(1)
                continue

            m = re.search(r'Build Date: (.*)$', line)
            if m:
                self.kernel_command_line = m.group(1)
                continue

            m = re.search(r'Log file: "(.*)", Time: (.*)$', line)
            if m:
                self.boot_logfile = m.group(1)
                self.boot_time = m.group(2)

            m = re.search(r'xorg-server ([^ ]+) .*$', line)
            if m:
                self.xserver_version = m.group(1)
                continue

            m = re.search(r'Using a default monitor configuration.', line)
            if m and self.xorg_conf_path is None:
                self.xorg_conf_path = 'default'
                continue

            m = re.search(r'Using config file: "(.*)"', line)
            if m:
                self.xorg_conf_path = m.group(1)
                continue

            # EDID and Modelines
            m = re.search(r'\(II\) (.*)\(\d+\): EDID for output (.*)', line)
            if m:
                self.displays[display_name] = display
                self.video_driver = m.group(1)
                display_name = m.group(2)
                display = {'Output': display_name}
                continue

            m = re.search(r'\(II\) (.*)\(\d+\): Assigned Display Device: (.*)$', line)
            if m:
                self.displays[display_name] = display
                self.video_driver = m.group(1)
                display_name = m.group(2)
                display = {'Output': display_name}
                continue

            m = re.search(r'Manufacturer: (.*) *Model: (.*) *Serial#: (.*)', line)
            if m:
                display['display manufacturer'] = m.group(1)
                display['display model']        = m.group(2)
                display['display serial no.']   = m.group(3)

            m = re.search(r'EDID Version: (.*)', line)
            if m:
                display['display edid version'] = m.group(1)

            m = re.search(r'EDID vendor \"(.*)\", prod id (.*)', line)
            if m:
                display['vendor'] = m.group(1)
                display['product id'] = m.group(2)

            m = re.search(r'Max Image Size \[(.*)\]: *horiz.: (.*) *vert.: (.*)', line)
            if m:
                display['size max horizontal'] = "%s %s" %(m.group(2), m.group(1))
                display['size max vertical'] = "%s %s" %(m.group(3), m.group(1))

            m = re.search(r'Image Size: *(.*) x (.*) (.*)', line)
            if m:
                display['size horizontal'] = "%s %s" %(m.group(1), m.group(3))
                display['size vertical'] = "%s %s" %(m.group(2), m.group(3))

            m = re.search(r'(.*) is preferred mode', line)
            if m:
                display['mode preferred'] = m.group(1)

            m = re.search(r'Modeline \"(\d+)x(\d+)\"x([0-9\.]+) *(.*)$', line)
            if m:
                key = "mode %sx%s@%s" %(m.group(1), m.group(2), m.group(3))
                display[key] = m.group(4)
                continue

            # Errors and Warnings
            m = re.search(r'\(WW\) (.*)$', line)
            if m:
                self.warnings.append(m.group(1))
                continue

            m = re.search(r'\(EE\) (.*)$', line)
            if m:
                self.errors.append(m.group(1))
                continue

            m = re.search(r'XINPUT: Adding extended input device "(.*:?)" \(type:\s+(.*:?)\)', line)
            if m:
                self.add_device(m.group(1), 'input', m.group(2))

        if display_name not in self.displays.keys():
            self.displays[display_name] = display
        in_file.close()


    def outputs_table(self):
        s = ''
        values = {}
        n = 0
        outputs = list(self.displays.keys())
        outputs.sort()
        for output in outputs:
            display = self.displays[output]
            if display == {}:
                continue
            for key in display.keys():
                val = display.get(key, '')
                if key not in values.keys():
                    values[key] = []
                    for i in range(0,n):
                        values[key].append('')
                values[key].append(val.strip())
            n += 1

        keys = list(values.keys())
        keys.sort()
        for key in keys:
            if key[:4] == 'mode':
                continue
            s += "%-30s " %(key)
            for val in values[key]:
                s += "%15s " %(val)
            s += "\n"

        return s

    def devices_table(self):
        s = ''
        for device in self.devices:
            s += "%-12s %-20s %-s\n" %(device['class'], device['name'], device['type'])
        return s

    def modules_table(self):
        s = ''
        s += "%-10s %-10s %5s %-30s %-s\n" %("Module", "Version", "ABI", "", "Vendor")
        lines = []
        for module in self.modules:
            abi_name = module['abi_name']
            lines.append("%-10s %-10s %5s %-30s %-s" %(
                module['name'], module['version'], module['abi_version'], module['abi_name'], module['vendor']))
        lines = list(set(lines))
        lines.sort()
        s += "\n".join(lines)
        s += "\n"
        return s

    def errors_filtered(self):
        excludes = set([
            'error, (NI) not implemented, (??) unknown.',
            'Failed to load module "fglrx" (module does not exist, 0)',
            'Failed to load module "nv" (module does not exist, 0)',
            ])
        return [err for err in self.errors if err not in excludes]

    def warnings_filtered(self):
        excludes = set([
            'warning, (EE) error, (NI) not implemented, (??) unknown.',
            'The directory "/usr/share/fonts/X11/cyrillic" does not exist.',
            'The directory "/usr/share/fonts/X11/100dpi/" does not exist.',
            'The directory "/usr/share/fonts/X11/75dpi/" does not exist.',
            'The directory "/usr/share/fonts/X11/100dpi" does not exist.',
            'The directory "/usr/share/fonts/X11/75dpi" does not exist.',
            'Warning, couldn\'t open module nv',
            'Warning, couldn\'t open module fglrx',
            'Falling back to old probe method for vesa',
            'Falling back to old probe method for fbdev',
            ])
        return [err for err in self.warnings if err not in excludes]


    def __str__(self):
        s = self.logfile + ":\n"
        s += "%15s : %s\n" %("Version", self.xserver_version)
        s += "%15s : %s\n" %("Boot Time", self.boot_time)
        s += "%15s : %s\n" %("Logfile", self.boot_logfile)
        s += "%15s : %s\n" %("Config File", self.xorg_conf_path)
        if self.video_driver is None:
            s += "%15s : Unknown\n" %("Video Driver")
        else:
            s += "%15s : %s\n" %("Video Driver", self.video_driver.lower())
        s += "%15s : %s\n" %("Kernel", self.kernel_version)
        s += "\n"
        s += self.modules_table()
        s += "\n"
        s += "Outputs:\n"
        s += self.outputs_table()
        s += "\n"
        s += "Devices:\n"
        s += self.devices_table()
        s += "\n"
        s += "Errors:\n"
        s += "\n".join(self.errors_filtered())
        s += "\n"
        s += "Warnings:\n"
        s += "\n".join(self.warnings_filtered())
        return s

    def process_record(self, text):
        print("TODO")

        self.add_device(device)
        self.add_module(module)

def loadfile(filename, binary=False):
    opentype = "r"
    if binary:
        opentype += "b"
    in_file = open(filename, opentype)
    text = in_file.read()
    in_file.close()
    return text

if __name__ == "__main__":
    import sys
    if len(sys.argv) < 2:
        xlog = XorgLog("/var/log/Xorg.0.log")
    else:
        xlog = XorgLog(sys.argv[1])
    print(xlog)

# TODO:  If size == 0x0, handle as a no-edid bug
# TODO:  If no mode preferred mentioned
# TODO:  If no modes found
# TODO:  If max size <10 or >80
# TODO:  Look up errors on http://www.x.org/wiki/FAQErrorMessages



#!/usr/bin/python3
# -*- coding: utf-8 -*-

def to_list(value):
    if type(value) is list:
        return value
    if value.endswith(','):
        value = value[:-1].strip()
    return [x.strip() for x in value.split(',')]

# vi:set ts=4 sw=4 expandtab:



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2010-2012 Bryce Harrington <bryce@canonical.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from __future__ import absolute_import, print_function, unicode_literals

'''High level package information'''
PROGNAME = 'xdiagnose'
VERSION = '3.0'
URL = 'http://launchpad.net/xdiagnose'
EMAIL = 'bryce@canonical.com'
DATE_STARTED = '2010-11-04'
DATE_COPYRIGHT = '2011'
LICENSE_URL = 'http://www.gnu.org/copyleft/gpl.html'

SHORT_DESCRIPTION = 'Analysis tools for troubleshooting X.org problems'

DESCRIPTION = """
This package is a friendly GUI application for diagnosing several
common X.org problems.
"""

class _contributor:
    '''Information about a person contributing to this project'''
    def __init__(self, name, email, started=None, roles=None, translating=None):
        self.name = name
        self.email = email
        self.started = started
        if roles is None:
            self.roles = []
        elif type(roles) is not list:
            self.roles = [roles]
        else:
            self.roles = roles
        self.translation_languages = translating
        return

    def to_dict(self):
        '''Returns the object in a dict suitable for json'''
        return self.__dict__

    @property
    def display_email(self):
        '''Formatted string version of email address'''
        if self.email:
            return '<%s>' % self.email
        else:
            return ''

    @property
    def display_roles(self):
        '''Formatted string version of roles list'''
        if self.roles:
            return '[%s]' % ','.join(self.roles)
        else:
            return ''

LEAD_DEVELOPER = _contributor(
    'Bryce Harrington', 'bryce@canonical.com', started='2010-11-04',
    roles=['lead', 'developer'], translating=None,
    )

CONTRIBUTORS = [
    _contributor(
        'Gabor Kelemen', 'kelemeng@ubuntu.com', started='2012-01-21',
        roles=['developer', 'translator'], translating=None),
    _contributor(
        'Jeff Lane', 'jeffrey.lane@canonical.com', started='2012-08-10',
        roles=['developer'], translating=None),
]


if __name__ == "__main__":
    print(PROGNAME, VERSION, URL)
    print("Copyright (C) %s %s <%s>" % (
        DATE_COPYRIGHT, LEAD_DEVELOPER.name, LEAD_DEVELOPER.email))
    print()
    for contributor in CONTRIBUTORS:
        print("%s %s %s" % (
            contributor.name,
            contributor.display_email,
            contributor.display_roles))



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2010-2012 Bryce Harrington <bryce@canonical.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import, print_function, unicode_literals

class Diagnostic:
    def __init__(self):
        self.name = None

    def run(self):
        pass

    def results(self):
        return "Results"

    def backup_xorg_conf(self):
        # xorg_conf_backup="/etc/X11/xorg.conf-backup-${timestamp}"
        # cp /etc/X11/xorg.conf ${xorg_conf_backup}
        print("Your config could not be backed up.")
        return False

    def default_config(self):
        if not self.backup_xorg_conf():
            return False

        # rm /etc/X11/xorg.conf
        #print "Failure restoring configuration to default."
        #print "Your config has not been changed."
        print("Your configuration has been restored to default and")
        print("your old configuration backed up.  Please restart.")
        return True

    def run_xorgconf(self):
        if not self.backup_xorg_conf():
            return False

        # Xorg :99 -configure
        print("Could not generate a new configuration")
        return False

    def view_xorg_log(self):
        # zenity --text-info --filename=$xorg_log --width=640 --height=480
        pass

    def view_gdm_log(self):
        # zenity --text-info --filename=${gdm_log_1} --width=640 --height=480
        pass

    def verify_xorgconf(self):
        # Run Alberto's xorg.conf checker (once it's available in main)
        print("Sorry, this option is not implemented yet")
        return False

    def edit_config(self):
        if not self.backup_xorg_conf():
            return False

        # xorg_conf_tmp=$(mktemp -t xorg.conf.XXXXXXXX)
        # zenity --text-info --editable --filename=/etc/X11/xorg.conf --width=640 --height=480 > "${xorg_conf_tmp}" && mv "${xorg_conf_tmp}" /etc/X11/xorg.conf
        # chmod 644 /etc/X11/xorg.conf

    def save_config_logs(self):
        pass
        #xorg_backup_name=failsafeX-backup-${timestamp}
        #xorg_backup_dir=$(mktemp -d -t ${xorg_backup_name}.XXX)
        #xorg_backup_file=/var/log/${xorg_backup_name}.tar

        # cp $xorg_conf $xorg_backup_dir
        #cp /etc/X11/xorg.conf $xorg_backup_dir
        #cp ${xorg_log} $xorg_backup_dir
        #cp ${xorg_log}.old $xorg_backup_dir
        #cp ${gdm_log} $xorg_backup_dir
        #cp ${gdm_log_1} $xorg_backup_dir
        #cp ${gdm_log_2} $xorg_backup_dir
        #lspci -vvnn > ${xorg_backup_dir}/lspci-vvnn.txt
        #xrandr --verbose > ${xorg_backup_dir}/xrandr-verbose.txt
        #tar -cf ${xorg_backup_file} ${xorg_backup_dir}
        #rm -rf ${xorg_backup_dir}

        #zenity --info --text "$(eval_gettext 'Relevant configuration and log files have been saved to:\n')"\$xorg_backup_file"\n$(gettext 'Bug reports can be submitted at http://www.launchpad.net/ubuntu/.\n')"


    def system_checkup(self):
        # * Look if user's video card pci id is not listed as supported by kernel or driver
        # * Look if -vesa or -fbdev has been loaded
        # * Look if glxgears shows software rendering
        # * Look if user has proprietary driver loaded
        # * Look for error messages in Xorg.0.log, dmesg, etc.
        #   LOG_ERRORS=$(grep -e "^(EE)" $xorg_log)
        # * Look if EDID information indicates something is incorrect
        # * Check if user has anything in xorg.conf and using a free driver
        pass

    def measure_performance(self):
        # Do some basic 2D and 3D operations to get some performance measurements
        pass

    def menu(self, level):
        if level == "0":
            print("Choice:")
            print("1. Run in low-graphics mode for just one session")
            print("2. Reconfigure graphics")
            print("3. Troubleshoot the error")
            print("4. Exit to console login")

        elif level == "1":
            pass

        elif level == "2":
            print("Reconfiguration:")
            print("How would you like to reconfigure your display?")
            print("2a. Use default (generic) configuration")
            print("2b. Create new configuration for this hardware")
            print("2c. Use your backed-up configuration")

        elif level == "3":
            print("Troubleshooting:")
            print("What information would you like to review?")
            print("3a. Review the xserver log file")
            print("3b. Review the startup errors")
            print("3c. Edit configuration file")
            print("3d. Archive configuration and logs")

        else:
            print("xdiagnose walks you through troubleshooting a few common X errors.")
            print()
            print(" * Screen blanks when lid is closed, and never comes back")
            print(" * 3D performance is horrible")
            print(" * Maximum resolution is lower than what screen can actually perform")

    def lid_close_blank(self):
        # Examine kernel logs to see if there was any evidence of it
        # Turn on storing of kernel logs from boot to boot
        # Tell user to reproduce the lid close blanking and then come back here
        # Once we know the cause, generate a quirk or whatnot for the issue
        pass

if __name__ == "__main__":

    diagnostic = Diagnostic()
    diagnostic.menu(0)



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2010-2012 Bryce Harrington <bryce@canonical.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import, print_function, unicode_literals

import sys
import os
import re
import binascii

from .utils.debug import (ERR, warn, dbg)
from .utils.file_io import (load_binary, load_file)


def _bytes(edid, byte1, byte2=None):
    if byte2:
        return edid[byte1*2:(byte2+1)*2]
    else:
        return edid[byte1*2:(byte1+1)*2]

def binary_to_bytecode(filename):
    data = load_binary(filename)
    return [ binascii.b2a_hex(data).decode("utf-8") ]

def load_edid_bytecodes(filename):
    """Loads a list of unique EDIDs from a given file.

    The file could be an Xorg.0.log with multiple bytecodes, or a binary
    edid retrieved from the monitor itself, or previously saved Edid data.
    Returns a list of bytecode strings, suitable for use with the Edid
    object constructor.
    """
    if (filename is None or filename == '' or not os.path.exists(filename)):
        raise Exception("Invalid filename %s" %(filename))

    try:
        # Try loading as a plain text file first (ala Xorg.0.log)
        lines = load_file(filename)
        if len(lines) < 1:
            raise Exception("Invalid file %s" %(filename))

        if lines[0].startswith('00ffffffffffff00'):
            # Looks like a regular edid file
            return [ lines.join("\n") ]

        # Next, assume it's an Xorg.0.log
        raw_edid = ""
        re_head = re.compile("\(II\) .*\(\d+\): EDID \(in hex\):$")
        re_edid = re.compile("\(II\) .*\(\d+\):\s\t([0-9a-f]{32})$")

        seen_edid_header = False
        edid_raw = ""
        edids = []
        # TODO: Make sure only unique edids are returned
        for line in lines:
            if re_head.search(line):
                seen_edid_header = True
            elif seen_edid_header:
                m = re_edid.search(line)
                if not m:
                    edids.append(edid_raw)
                    edid_raw = ""
                    seen_edid_header = False
                    continue
                edid_raw += m.group(1)
        return edids

    except UnicodeDecodeError:
        return binary_to_bytecode(filename)

    return None


class EdidFirmware(object):
    EDID_FIRMWARE_PATH = '/lib/firmware/edid'
    EDID_DRM_CONF_PATH = '/etc/modprobe.d/drm-kms-helper.conf'

    def __init__(self):
        pass

    def list(self):
        for filename in os.listdir(self.EDID_FIRMWARE_PATH):
            edid_path = os.path.join(self.EDID_FIRMWARE_PATH, filename)
            lines = binary_to_bytecode(edid_path)
            edid = Edid("\n".join(lines))
            edid._origin = "firmware"
            yield edid

    def install(self, edid_filename):
        '''Installs the named edid file into the firmware directory'''
        import errno
        import shutil
        try:
            os.makedirs(self.EDID_FIRMWARE_PATH)
        except OSError as exc:
            if exc.errno != errno.EEXIST:
                warn("Could not mkdir %s" %(self.EDID_FIRMWARE_PATH))
                return False

        # Install the EDID
        edid_firmware = os.path.basename(edid_filename)
        try:
            target = os.path.join(self.EDID_FIRMWARE_PATH, edid_firmware)
            shutil.copyfile(edid_filename, target)
            print("Installed %s" %(target))
            return True
        except:
            warn("Could not install firmware")
            raise
            return False

    def uninstall(self, edid_filename):
        '''Uninstalls the named edid file from the firmware directory'''
        edid_firmware = os.path.basename(edid_filename)
        try:
            target = os.path.join(self.EDID_FIRMWARE_PATH, edid_firmware)
            os.remove(target)
            print("Uninstalled %s" %(target))
            return True
        except:
            warn("Could not uninstall %s" %(target))
            raise
            return False

    def activate(self, edid_name):
        '''Activates given edid by passing it as a kernel command line parameter'''

        # TODO: Install it automatically?
        #self._install(edid_name)
        f = open(self.EDID_DRM_CONF_PATH, 'w')
        f.write("options drm_kms_helper edid_firmware=edid/%s\n" %(edid_name))
        f.close()
        print("Activated %s via %s" %(edid_name, self.EDID_DRM_CONF_PATH))
        return True

    def deactivate(self, edid_name):
        line_to_remove = "options drm_kms_helper edid_firmware=edid/%s" %(edid_name)
        f = open(self.EDID_DRM_CONF_PATH, 'w')
        lines = f.readlines()
        f.close()
        f = open(self.EDID_DRM_CONF_PATH, 'w')
        for line in lines:
            if line != line_to_remove:
                f.write(line)
        f.close()

class Edid(object):
    def __init__(self, bytecode=None):
        '''bytecode: multiline hexadecimal text such as from an Xorg.0.log'''
        self._items = None
        self._origin = None
        self.edid_raw = bytecode
        if self.edid_raw is not None:
            # TODO: Move this to top
            assert bytecode.startswith('00ffffffffffff00'), "bytecode is not valid EDID data"
            self._origin = "custom"

    def save(self, filename):
        file = open(filename, 'wb')
        file.write(self.to_binary())
        file.close()
        return True

    def _parse(self, edid):
        if edid is None:
            return None
        return [
            ("Header", _bytes(edid,0,7)),
            ("Manufacturer", _bytes(edid,8,9)),
            ("Product ID code", _bytes(edid,10,11)),
            ("Serial Number", _bytes(edid,12,15)),
            ("Week of Manufacture", _bytes(edid,16)),
            ("Year of Manufacture", _bytes(edid,17)),
            ("EDID Version", _bytes(edid,18)),
            ("EDID Revision", _bytes(edid,19)),

            ("Video input def", _bytes(edid,20)),
            ("Max Horiz Image(cm)", _bytes(edid,21)),
            ("Max Vert Image(cm)", _bytes(edid,22)),
            ("Gamma", _bytes(edid,23)),
            ("Power management", _bytes(edid,24)),

            ("Chromaticity", _bytes(edid,25,34)),
            ("Timing I", _bytes(edid,35)),
            ("Timing II", _bytes(edid,36)),
            ("Reserved Timing", _bytes(edid,37)),
            ("Standard Timing", _bytes(edid,38,53)),
            ("Horiz Active (px)", _bytes(edid,56)),
            ("Horiz Blanking", _bytes(edid,57)),
            ("Horiz high", _bytes(edid,58)),
            ("Vert Active", _bytes(edid,59)),
            ("Vert Blank", _bytes(edid,60)),
            ("Vert high", _bytes(edid,61)),
            ("Horz Sync Offset (px)", _bytes(edid,62)),
            ("Horiz Sync Pulse Width (px)", _bytes(edid,63)),
            ("Vert Sync (lines)", _bytes(edid,64)),
            ("high", _bytes(edid,65)),
            ("Horiz Image Size (mm)", _bytes(edid,66)),
            ("Vert Image Size (mm)", _bytes(edid,67)),
            ("Image Size high", _bytes(edid,68)),
            ("Horiz Border", _bytes(edid,69)),
            ("Vert Border", _bytes(edid,70)),
            ("Interlacing", _bytes(edid,71)),
            ("Descriptor Block 2", _bytes(edid,72,89)),
            ("Descriptor Block 3", _bytes(edid,90,107)),
            ("Descriptor Block 4", _bytes(edid,108,125)),
            ("Extension Flag", _bytes(edid,126)),
            ("Checksum", _bytes(edid,127)),
            ]

    def to_hex(self):
        return self.edid_raw

    def to_binary(self):
        return bytes.fromhex(self.edid_raw)

    @property
    def items(self):
        if self._items is None:
            self._items = self._parse(self.to_hex())
        return self._items

    @property
    def manufacturer(self):
        code = self.items[1][1]              # The ascii code
        bstr = str(bin(int(code, 16)))[2:]   # Converted to binary
        bstr = bstr.zfill(15)                # Fill left side with 0's
        s = [                                # Extract and convert letter codes to chars
            chr(int(bstr[-15:-10],2) + ord('A') - 1),
            chr(int(bstr[-10:-5],2) + ord('A') - 1),
            chr(int(bstr[-5:],2) + ord('A') - 1)
            ]
        return ''.join(s)

    @property
    def product_id(self):
        return _bytes(self.to_hex(),10,11)

    @property
    def serial_number(self):
        return _bytes(self.to_hex(),12,15)

    @property
    def name(self):
        return "%s:%s  SN#%s  %d-%d  v%s.%s  %s" %(
            self.manufacturer,
            self.items[2][1],
            self.items[3][1],
            int(self.items[4][1], 16),
            int(self.items[5][1], 16) + 1990,
            self.items[6][1],
            self.items[7][1],
            self._origin
            )

    def __str__(self):
        text = ''
        for field, value in self.items:
            text += "%-30s: %s\n" %(field, value)
        return text


if __name__ == "__main__":
    edidfile = sys.argv[1]
    regex = re.compile("\(II\) .*\(\d+\):\s+(.*)$")

    edid_text = ""
    lines = load_file(edidfile)
    for line in lines.split("\n"):
        m = regex.search(line)
        if m:
            line = m.group(1)
        edid_text += line

    edid = Edid(edid_text)
    print(edid)



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2011-2012 Bryce Harrington <bryce@bryceharrington.org>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

'''Routines to encode or convert to and from text'''

from __future__ import absolute_import, print_function, unicode_literals

from decimal                     import Decimal

def quote(msg):
    """
    Similar to urllib.quote but for glibs GMarkup
    @param msg: string to quote
    @returns: quoted string
    """
    msg = msg.replace('&', '&amp;')
    msg = msg.replace('<', '&lt;')
    msg = msg.replace('>', '&gt;')
    return msg

def o2str(obj):
    """
    Convert a unicode, decimal.Decimal, datetime object, etc. to a str.
    Converts lists and tuples of objects into lists of strings.
    """
    retval = None
    if type(obj) == str:
        return obj
# Type 'unicode' no longer exists in python3
#    elif type(obj) == unicode:
#        return obj.encode('ascii', 'ignore')
    elif type(obj) == Decimal:
        return str(obj)
    elif type(obj) == list or type(obj) is tuple:
        new_list = []
        for item in obj:
            new_list.append(o2str(item))
        return new_list
    elif str(type(obj)) == "<type 'datetime.datetime'>":
        return obj.ctime()
    else:
        #print str(type(obj))
        return obj

def to_bool(value):
    """
    Converts 'something' to boolean. Raises exception for invalid formats
    Possible True  values: 1, True, '1', 'TRue', 'yes', 'y', 't'
    Possible False values: 0, False, None, [], {}, '', '0', 'faLse', 'no', 'n', 'f', 0.0
    """
    if type(value) == type(''):
        if value.lower() in ("yes", "y", "true",  "t", "1"):
            return True
        if value.lower() in ("no",  "n", "false", "f", "0", "none", ""):
            return False
        raise Exception('Invalid value for boolean conversion: ' + value)
    return bool(value)

def o2float(value):
    '''Converts strings like 42%, 123M, 1.2B into floating point numbers

    Returned values are in millions, so '1.2B' returns 1200
    '''
    if value is None:
        return 0.0
    elif type(value) is float:
        return value
    elif type(value) is int:
        return float(value)
    elif value == '--':
        return 0.0

    value = value.replace(',','')
    last = value[len(value)-1]
    if last == 'M':
        return float(value[:-1])
    elif last == 'B':
        return float(value[:-1]) * 1000
    elif last == '%':
        return float(value[:-1])/100.0
    elif last == ')' and value[0] == '(':
        return -1 * o2float(value[1:-1])

    try:
        return float(value)
    except ValueError:
        sys.stderr.write("ofloat: Could not convert '%s' to float\n" %(value))
        raise


if __name__ == "__main__":
    test_cases = [
        ('true', True),
        ('t', True),
        ('yes', True),
        ('y', True),
        ('1', True),
        ('false', False),
        ('f', False),
        ('no', False),
        ('n', False),
        ('0', False),
        ('', False),
        (1, True),
        (0, False),
        (1.0, True),
        (0.0, False),
        ([], False),
        ({}, False),
        ((), False),
        ([1], True),
        ({1:2}, True),
        ((1,), True),
        (None, False),
        (object(), True),
        ]
    for test, expected in test_cases:
        assert to_bool(test) == expected, "to_bool("+test+") failed to return "+expected



#!/usr/bin/python3

from __future__ import absolute_import, print_function, unicode_literals

import os
import os.path
import sys
import re
import datetime

from apport.hookutils import *

from apport import unicode_gettext as _

pci_devices = [
    { 'name':'i810',        're':'(8086:7121)',        'supported':False },
    { 'name':'i810dc',      're':'(8086:7123)',        'supported':False },
    { 'name':'i810e',       're':'(8086:7125)',        'supported':False },
    { 'name':'i815',        're':'(8086:1132|82815)',  'supported':False },
    { 'name':'i830',        're':'(8086:3577|82830)',  'supported':False },
    { 'name':'i845',        're':'(8086:2562|82845G)', 'supported':False },
    { 'name':'i855',        're':'(8086:3582|855GM)',  'supported':False },
    { 'name':'i865',        're':'(8086:2572|82865G)', 'supported':False },
    { 'name':'i915g',       're':'(8086:2582)'       },
    { 'name':'i915gm',      're':'(8086:2592|915GM)' },
    { 'name':'e7221',       're':'(8086:258a)',        'supported':False },
    { 'name':'i945g',       're':'(8086:2772|945G[ \/]|82945G[ \/])' },
    { 'name':'i945gm',      're':'(8086:27a2|945GM[ \/]|82945GM[ \/])' },
    { 'name':'i945gme',     're':'(8086:27ae|945GME|82945GME)' },
    { 'name':'IGDg',        're':'(8086:a001)' },
    { 'name':'IGDgm',       're':'(8086:a011)' },
    { 'name':'pineviewg',   're':'(8086:a001)' },
    { 'name':'pineviewgm',  're':'(8086:a011)' },
    { 'name':'i946gz',      're':'(8086:2972|82946GZ)' },
    { 'name':'g35',         're':'(8086:2982|82G35)' },
    { 'name':'i965q',       're':'(8086:2992|Q965)' },
    { 'name':'i965g',       're':'(8086:29a2|G965)' },
    { 'name':'g33',         're':'(8086:29c2|82G33)' },
    { 'name':'q35',         're':'(8086:29b2)' },
    { 'name':'q33',         're':'(8086:29d2)' },
    { 'name':'i965gm',      're':'(8086:2a02|GM965)' },
    { 'name':'i965gme',     're':'(8086:2a12)' },
    { 'name':'gm45',        're':'(8086:2a42)' },
    { 'name':'IGDeg',       're':'(8086:2e02)' },
    { 'name':'q45',         're':'(8086:2e12)' },
    { 'name':'g45',         're':'(8086:2e22)' },
    { 'name':'g41',         're':'(8086:2e32)' },
    { 'name':'b43',         're':'(8086:2e42)' },
    { 'name':'clarkdale',   're':'(8086:0042)' },
    { 'name':'arrandale',   're':'(8086:0046)' },
    { 'name':'sandybridge-gt1',   're':'(8086:0102)' },
    { 'name':'sandybridge-m-gt1', 're':'(8086:0106)' },
    { 'name':'sandybridge-s',     're':'(8086:010a)' },
    { 'name':'sandybridge-gt2',   're':'(8086:0112)' },
    { 'name':'sandybridge-m-gt2', 're':'(8086:0116)' },
    { 'name':'sandybridge-gt2+',  're':'(8086:0122)' },
    { 'name':'sandybridge-m-gt2+','re':'(8086:0126)' },
    ]
for device in pci_devices:
    device['rc'] = re.compile(device['re'], re.IGNORECASE)

def get_pci_device(text):
    regex_vga = re.compile('VGA compatible controller (.*)', re.IGNORECASE)

    lines = regex_vga.findall(text)
    if len(lines) > 0:
        for l in lines:
            if len(l.strip())>0:
                for device in pci_devices:
                    if device['rc'].search(l.strip()):
                        return device
    return None

def get_signature(text):
    '''Assumes the format of the i915_error_state file'''
    codes = {
        'EIR' :      re.compile("EIR: 0x([0-9a-fA-F]+)"),
        'ESR' :      re.compile("ESR: 0x([0-9a-fA-F]+)"),
        'PGTBL_ER' : re.compile("PGTBL_ER: 0x([0-9a-fA-F]+)"),
        'IPEHR' :    re.compile("IPEHR: 0x([0-9a-fA-F]+)"),
        }
    section = None
    signature = ''

    for line in text.split("\n"):
        if line == 'Blitter command stream:':
            section = 'blitter'
        elif line == 'Video (BSD) command stream:':
            section = 'video'
        elif line == 'Render command stream:':
            section = 'render'
        elif line[:8] == 'Active [':
            section = 'active'
            return signature
        else:
            for k, r in codes.items():
                match = r.search(line)
                if match and match.group(1) != "00000000":
                    if section:
                        signature += " %s.%s: 0x%s" %(section, k, match.group(1))
                    else:
                        signature += " %s: 0x%s" %(k, match.group(1))

    return signature

def main(argv=None):
    if argv is None:
        argv = sys.argv

    if '--force' not in argv:
        # If not a development release, bail
        rel = command_output(['lsb_release', '-d'])
        if 'development branch' not in rel:
            return 1

        from apport.packaging_impl import impl as packaging
        if not packaging.enabled():
            return 2

    import apport.report
    report = apport.report.Report(type='Crash')
    report.setdefault('Tags', '')
    report.setdefault('Title', 'GPU lockup')

    report.add_os_info()
    report.add_proc_info()
    report.add_user_info()

    report['Package'] = 'xserver-xorg-video-intel'
    report['Tags'] += ' freeze'
    report['Lspci'] = command_output(['lspci', '-vvnn'])
    device = get_pci_device(report['Lspci'])
    if device and 'name' in device:
        if '--force' not in argv:
            if 'supported' in device and device['supported'] == False:
                # Unsupported chipset; we don't want bugs reported for this HW
                return 3
        report['Chipset'] = device['name']
        report['Title'] = "[%s] GPU lockup" %(device['name'])

    try:
        attach_hardware(report)
        attach_related_packages(report, ["xserver-xorg", "libdrm2", "xserver-xorg-video-intel"])
    except:
        # attach_hardware can fail with UnicodeDecodeError parsing DMI (See LP: #1062042)
        # attach_related_packages can fail with SystemError if apt isn't updated (See LP: #1103061)
        pass
    attach_file_if_exists(report, '/etc/X11/xorg.conf', 'XorgConf')
    attach_file(report, '/var/log/Xorg.0.log', 'XorgLog')
    attach_file_if_exists(report, '/var/log/Xorg.0.log.old', 'XorgLogOld')
    attach_file_if_exists(report, '/sys/kernel/debug/dri/0/i915_error_state', 'i915_error_state')

    signature = get_signature(report.get('i915_error_state', ''))
    if not signature:
        # Lack of a signature generally indicates an invalid error state, such
        # as a false positive or a '0x000000' (non-)error state.  In either case,
        # the bug won't be upstreamable nor otherwise actionable.
        if '--force' not in argv:
            return 4

    report['Title'] += " " + signature
    report['DuplicateSignature'] = "%s %s" %(report['Title'], report['DistroRelease'])

    nowtime = datetime.datetime.now()
    report_filename = '/var/crash/%s.%s.crash' % (report['Package'], str(nowtime).replace(' ', '_'))

    if '--stdout' in argv:
        print("# %s" %(report_filename))
        report.write(sys.stdout.buffer)
        return 0

    report_file = os.fdopen(os.open(report_filename, os.O_WRONLY|os.O_CREAT|os.O_EXCL), 'wb')
    os.chmod(report_filename, 0o600)

    try:
        report.write(report_file)
    finally:
        report_file.close()
    return 0

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2011-2012 Bryce Harrington <bryce@bryceharrington.org>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import, print_function, unicode_literals

from exceptions import Exception
from time import sleep
from subprocess import (Popen, PIPE)

from debug import (stderr_msg, dbg, ERR)

# TODO: Make *_with_input be aliases calling the base routines
# TODO: Integrate use of dbg and ERR from utils

class ReturnCode(Exception):
    def __init__(self, code, errors=None):
        self.code = code
        if type(errors) in (list, tuple):
            self.errors = errors
        else:
            self.errors = [errors]

    def __str__(self):
        text = '\n'.join(self.errors)
        return "%sReturned error code %d" %(text, self.code)


def shell(command):
    """Executes command in a shell, returns stdout; prints errors to stderr"""
    dbg("shell: %s" %(' '.join(command)))
    p = Popen(command, shell=True, stdout=PIPE, stderr=PIPE)
    output = "\n".join(p.stdout.readlines())
    if p.returncode:
        raise ReturnCode(p.returncode, p.stderr.readlines())
    return output

def shell_with_input(command, in_text):
    dbg("shell_with_input: %s" %(' '.join(command)))
    p = Popen(command, shell=True, stdout=PIPE, stderr=PIPE, stdin=PIPE)
    output, stderr = p.communicate(input=in_text)
    if p.returncode:
        raise ReturnCode(p.returncode, stderr)
    return output

def execute(command, in_text=None):
    """Executes command, returns stdout; prints errors to stderr"""
    dbg("execute: `%s`" %(' '.join(command)))
    if in_text is None:
        p = Popen(command, shell=False, stdout=PIPE, stderr=PIPE)
    else:
        p = Popen(command, shell=False, stdout=PIPE, stderr=PIPE, stdin=PIPE)
        dbg("execute: polling (%s)..." %(in_text))
        while p.poll() is None and p.stdin is not None:
            dbg("execute: Sending to process stdin")
            p.stdin.write(in_text)
            dbg("execute: sleeping")
            sleep(0.01)
    output = p.stdout.read()
    if p.returncode:
        dbg("Received return code %d" %(p.returncode))
        raise ReturnCode(p.returncode, p.stderr.readlines())
    return output

def execute_with_input(command, in_text):
    """Executes command, passing in_text to stdin if provided"""
    execute(command, in_text)

def is_X_running():
    # TODO: Reuse one of the above commands
    p = Popen(["xset", "-q"], stdout=PIPE, stderr=PIPE)
    p.communicate()
    if p.returncode != 0:
        print("Error")
    return p.returncode == 0



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2010-2012 Bryce Harrington <bryce@canonical.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import os
import sys
import stat
import subprocess
import shutil
from gi.repository import Gtk
from gi.repository.GdkPixbuf import Pixbuf
import re
import tempfile
import gettext
from gettext import gettext as _

from xdiagnose.config_update import (
    config_update,
    config_dict,
    safe_backup,
    )
from xdiagnose.errors_treeview import ErrorsTreeView

class XDiagnoseApplet(Gtk.Window):
    def __init__(self):
        self.__enable_debugging = None
        self.__enable_apport = None
        self.__disable_splash = None
        self.__disable_vesafb = None
        self.__disable_pat = None
        self.__disable_grub_graphics = None
        self.__grub_gfxpayload_linux = None
        self.__grub_config_filename = '/etc/default/grub'
        self.kparams = {}
        self.is_running = True

        if not os.path.exists(self.__grub_config_filename):
            print("Error: No %s present" %(self.__grub_config_filename))
            sys.exit(1)

        Gtk.Window.__init__(self)
        self.set_title(_("X Diagnostics Settings"))
        self.set_size_request(300,200)
        self.set_border_width(10)
        self.connect("delete-event", self.on_close)
        self.connect("destroy", self.on_close)

        vbox = Gtk.VBox()
        vbox.set_spacing(20)
        self.add(vbox)

        vbox.pack_start(self.build_toolbar(), True, True, 0)
        vbox.pack_start(self.build_settings_list(
            title=_("Debug"), settings=[
                {'text':_("Extra graphics _debug messages"),
                 'tip':_("Makes dmesg logs more verbose with details about 3d, plymouth, and monitor detection"),
                 'active':self.has_enable_debugging(),
                 'handler':self.handle_enable_debugging},
                {'text':_("Display boot messages"),
                 'tip':_("Removes splash and quiet from kernel options so you can see kernel details during boot"),
                 'active':self.has_disable_splash(),
                 'handler':self.handle_disable_splash},
                {'text':_("Enable automatic crash bug reporting"),
                 'tip':_("Turns on the Apport crash detection and bug reporting tool"),
                 'active':self.has_enable_apport(),
                 'handler':self.handle_enable_apport},
                ]), True, True, 0)
        vbox.pack_start(self.build_settings_list(
            title=_("Workarounds"), settings=[
                {'text':_("Disable bootloader _graphics"),
                 'tip':_("The grub bootloader has a graphics mode using the VESA framebuffer driver which can sometimes interfere with later loading of the proper video driver.  Checking this forces grub to use text mode only."),
                 'active':self.has_disable_grub_graphics(),
                 'handler':self.handle_disable_grub_graphics},
                {'text':_("Disable _VESA framebuffer driver"),
                 'tip':_("vesafb is loaded early during boot so the boot logo can display, but can cause issues when switching to a real graphics driver.  Checking this prevents vesafb from loading so these issues do not occur."),
                 'active':self.has_disable_vesafb(),
                 'handler':self.handle_disable_vesafb},
                {'text':_("Disable _PAT memory"),
                 'tip':_("This pagetable extension can interfere with the memory management of proprietary drivers under certain situations and cause lagging or failures to allocate video memory, so turning it off can prevent those problems."),
                 'active':self.has_disable_pat(),
                 'handler':self.handle_disable_pat},
                ]), True, True, 0)

        button_box = Gtk.HBox()
        button_box.set_spacing(20)
        vbox.add(button_box)

        apply_button = Gtk.Button("Apply")
        apply_button.connect('clicked', self.on_apply)
        button_box.add(apply_button)

        close_button = Gtk.Button("Close")
        close_button.connect('clicked', self.on_close)
        button_box.add(close_button)

        self.show_all()

        # Where is our data dir?
        datadir = None
        for d in [
            os.path.relpath(os.path.join(os.path.dirname(__file__), "../data")),
            "/usr/local/share/xdiagnose",
            "/usr/share/xdiagnose"]:
            if os.path.exists(d):
                datadir = d
                break

        # Icons
        icon_file = os.path.join(datadir, 'icons', 'microscope.svg')
        pixbuf = Pixbuf.new_from_file(icon_file)
        self.set_icon(pixbuf)

    def destroy(self, widget=None, event=None, data=None):
        self.is_running = False
        self.destroy()
        return False

    def build_toolbar(self):
        hbox = Gtk.HBox()
        hbox.set_spacing(10)

        b = Gtk.Button(_("View Errors"))
        b.connect('clicked', self.on_scan_errors)
        hbox.pack_start(b, False, False, 0)

        b = Gtk.Button(_("Report an Xorg Bug"))
        b.connect('clicked', self.on_report_bug_action)
        hbox.pack_start(b, False, False, 0)

        return hbox

    def build_settings_list(self, title, settings=None):
        frame = Gtk.Frame()
        frame.set_shadow_type(Gtk.ShadowType.NONE)

        label = Gtk.Label(label="<b>%s</b>" %(title))
        label.set_use_markup(True)
        frame.set_label_widget(label)

        alignment = Gtk.Alignment.new(0.5, 0.5, 1.0, 1.0)
        alignment.set_padding(5, 0, 12, 0)

        vbox = Gtk.VBox()
        for s in settings:
            checkbutton = Gtk.CheckButton(s['text'], use_underline=True)
            checkbutton.connect("toggled", s['handler'])
            if 'tip' in s and s['tip']:
                checkbutton.set_tooltip_text(s['tip'])
            if 'active' in s:
                if s['active']:
                    checkbutton.set_active(1)
                else:
                    checkbutton.set_active(0)
            if 'inconsistent' in s and s['inconsistent']:
                checkbutton.set_inconsistent(s['inconsistent'])
            vbox.pack_start(checkbutton, False, False, 0)

        alignment.add(vbox)
        frame.add(alignment)
        return frame

    def load_config(self):
        assert(os.path.exists(self.__grub_config_filename))
        d = config_dict(self.__grub_config_filename)
        self.kparams = {}
        if 'GRUB_CMDLINE_LINUX_DEFAULT' in d:
            re_kparam = re.compile("^([\w\.]+)=(.*)")
            kparam_str = d['GRUB_CMDLINE_LINUX_DEFAULT'].replace('"', '')
            for param in kparam_str.split(' '):
                value = 1
                m = re_kparam.match(param)
                if m:
                    param = m.group(1)
                    value = m.group(2)
                self.kparams[param] = value
        if 'GRUB_GFXPAYLOAD_LINUX' in d:
            re_kparam = re.compile("^([\w\.]+)=(.*)")
            value = d['GRUB_GFXPAYLOAD_LINUX'].replace('"', '')
            self.__grub_gfxpayload_linux = value
            if value == 'text':
                self.has_disable_grub_graphics(True)
            else:
                self.has_disable_grub_graphics(False)
        else:
            self.has_disable_grub_graphics(False)

        if 'drm.debug' in self.kparams:
            self.has_enable_debugging(True)
        else:
            self.has_enable_debugging(False)

        if 'splash' not in self.kparams:
            self.has_disable_splash(True)
        else:
            self.has_disable_splash(False)

        if 'vesafb.invalid' in self.kparams:
            self.has_disable_vesafb(True)
        else:
            self.has_disable_vesafb(False)

        if 'nopat' in self.kparams:
            self.has_disable_pat(True)
        else:
            self.has_disable_pat(False)

    def load_apport_config(self):
        path = "/etc/default/apport"
        if not os.path.exists(path):
            return

        d = config_dict(path)
        apport_enabled = d.get('enabled', None)
        if apport_enabled == '1':
            self.has_enable_apport(True)
        elif apport_enabled == '0':
            self.has_enable_apport(False)

    def has_enable_debugging(self, value=None):
        if value is not None:
            self.__enable_debugging = value
        elif self.__enable_debugging is None:
            self.load_config()
        return self.__enable_debugging
    def handle_enable_debugging(self, widget):
        self.has_enable_debugging(widget.get_active())

    def has_disable_splash(self, value=None):
        if value is not None:
            self.__disable_splash = value
        elif self.__disable_splash is None:
            self.load_config()
        return self.__disable_splash
    def handle_disable_splash(self, widget):
        self.has_disable_splash(widget.get_active())

    def has_disable_vesafb(self, value=None):
        if value is not None:
            self.__disable_vesafb = value
        elif self.__disable_vesafb is None:
            self.load_config()
        return self.__disable_vesafb
    def handle_disable_vesafb(self, widget):
        self.has_disable_vesafb(widget.get_active())

    def has_disable_pat(self, value=None):
        if value is not None:
            self.__disable_pat = value
        elif self.__disable_pat is None:
            self.load_config()
        return self.__disable_pat
    def handle_disable_pat(self, widget):
        self.has_disable_pat(widget.get_active())

    def has_enable_apport(self, value=None):
        if value is not None:
            self.__enable_apport = value
        elif self.__enable_apport is None:
            self.load_apport_config()
        return self.__enable_apport
    def handle_enable_apport(self, widget):
        active = widget.get_active()
        if active == self.__enable_apport:
            return
        print("Handling apport enabled")

        if self.has_enable_apport(widget.get_active()):
            enabled = 1
        else:
            enabled = 0

        filename = "/etc/default/apport"
        fd, temp_path = tempfile.mkstemp(text=True)
        os.chmod(temp_path, stat.S_IWUSR | stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH)
        fo = os.fdopen(fd, "w+")
        config_update(filename, override_params={'enabled': enabled}, fileio=fo)
        fo.close()

        # Move new file into place
        shutil.move(temp_path, filename)
        os.chmod(filename, stat.S_IWUSR | stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH)


    def has_disable_grub_graphics(self, value=None):
        if value is not None:
            self.__disable_grub_graphics = value
        elif self.__disable_grub_graphics is None:
            self.load_config()
        return self.__disable_grub_graphics
    def handle_disable_grub_graphics(self, widget):
        self.has_disable_grub_graphics(widget.get_active())

    def update_grub(self):
        try:
            subprocess.call(['/usr/sbin/update-grub'])
            return True
        except:
            # TODO: Indicate error occurred
            return False

    def on_report_bug_action(self, widget):
        process = subprocess.Popen(['ubuntu-bug', 'xorg'])
        process.communicate()

    def on_scan_errors(self, widget):
        re_xorg_error = re.compile("^\[\s*([\d\.]+)\] \(EE\) (.*)$")
        re_dmesg_error = re.compile("^\[\s*(\d+\.\d+)\] (.*(?:BUG|ERROR|WARNING).*)$")
        re_jockey_error = re.compile("^(\d+\-\d+-\d+ \d+:\d+:\d+,\d+) ERROR: (.*)$")
        errors = []

        # Xorg.0.log errors
        process = subprocess.Popen(['grep', '(EE)', '/var/log/Xorg.0.log'], universal_newlines=True)
        stdout, stderr = process.communicate()
        for err in str(stdout).split("\n"):
            m = re_jockey_error.match(err)
            if not m:
                continue
            timestamp = m.group(1)
            errmsg = m.group(2)
            errors.append(errmsg)

        # dmesg errors
        process = subprocess.Popen(['dmesg'], universal_newlines=True)
        stdout, stderr = process.communicate()
        for err in str(stdout).split("\n"):
            m = re_dmesg_error.match(err)
            if not m:
                continue
            timestamp = m.group(1)
            errmsg = m.group(2)
            errors.append(errmsg)

        # jockey errors
        if os.path.exists('/var/log/jockey.log'):
            process = subprocess.Popen(['grep', 'ERROR', '/var/log/jockey.log'], universal_newlines=True)
            stdout, stderr = process.communicate()
            for err in str(stdout).split("\n"):
                m = re_dmesg_error.match(err)
                if not m:
                    continue
                timestamp = m.group(1)
                errmsg = m.group(2)
                errors.append(errmsg)

        errors_window = ErrorsTreeView(errors)

    def on_apply(self, widget):
        overrides = {}
        merges = {}
        cmdline_default_params = self.kparams
        fd, temp_grub_path = tempfile.mkstemp(text=True)
        os.chmod(temp_grub_path, stat.S_IWUSR | stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH)
        fo = os.fdopen(fd, "w+")
        if self.__disable_splash:
            if 'quiet' in cmdline_default_params:
                del cmdline_default_params['quiet']
            if 'splash' in cmdline_default_params:
                del cmdline_default_params['splash']
        else:
            cmdline_default_params['quiet'] = None
            cmdline_default_params['splash'] = None
        if self.__disable_pat:
            cmdline_default_params['nopat'] = None
        elif 'nopat' in cmdline_default_params:
            del cmdline_default_params['nopat']
        if self.__disable_vesafb:
            cmdline_default_params['vesafb.invalid'] = 1
        elif 'vesafb.invalid' in cmdline_default_params:
            del cmdline_default_params['vesafb.invalid']
        if self.__enable_debugging:
            # TODO: Enable debug for Xorg.0.log
            cmdline_default_params['drm.debug'] = '0xe'
            cmdline_default_params['plymouth:debug'] = None
        elif 'drm.debug' in cmdline_default_params:
            del cmdline_default_params['drm.debug']
        if self.__disable_grub_graphics is not None:
            # NOTE: Where text graphics works, this can be handled by blacklisting in grub.
            #       Should offer to file a request for this against grub maybe.
            if self.__disable_grub_graphics:
                overrides['GRUB_GFXPAYLOAD_LINUX'] = 'text'
            elif self.__grub_gfxpayload_linux != 'text':
                overrides['GRUB_GFXPAYLOAD_LINUX'] = self.__grub_gfxpayload_linux
            else:
                overrides['GRUB_GFXPAYLOAD_LINUX'] = 'keep'

        kparams = []
        for k,v in cmdline_default_params.items():
            if not k:
                continue
            elif v is None:
                kparams.append(k)
            else:
                kparams.append("%s=%s" %(k, v))
        overrides['GRUB_CMDLINE_LINUX_DEFAULT'] = '"%s"' %(' '.join(kparams))

        assert(os.path.exists(self.__grub_config_filename))
        config_update(self.__grub_config_filename, override_params=overrides, merge_params=merges, fileio=fo)

        fo.close()

        # Backup the old file
        try:
            bak_path = safe_backup(self.__grub_config_filename)
            if not bak_path:
                # TODO: Display error message dialog
                print("Error:  Could not backup file %s.  Changes not applied." %(
                    self.__grub_config_filename))
                return
        except IOError as err:
            # TODO: Display error message dialog
            print("Error:  Failure creating backup file for %s.  Changes not applied." %(
                self.__grub_config_filename))
            print(err)
            return

        # Move new file into place
        shutil.move(temp_grub_path, self.__grub_config_filename)
        os.chmod(self.__grub_config_filename, stat.S_IWUSR | stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH)

        # Finally, update grub
        # TODO: Only do this if we change any of the grub parameters
        self.update_grub()

        # TODO: Mark Apply button insensitive

    def run(self):
        Gtk.main()

    def on_close(self, widget=None, event=None, data=None):
        Gtk.main_quit()
        return False


# TODO: cmdline option to display grub file contents (--dryrun?)



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2010-2012 Bryce Harrington <bryce@canonical.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# Required dependency for:
#  + xorg-pkg-tools
#  + upstreamer
#  + upstream-versions
#  + xgit-checkout

import re

def compile_table(table):
    for regex in table:
        regex['rc'] = re.compile(regex['re'], re.IGNORECASE)

def invert_dict(d):
    return dict([[v,k] for k,v in d.items()])

def group_match_capitalize(match):
    return match.group(1).capitalize()

def group_match_lower(match):
    return match.group(1).lower()

deb_to_fdo_mapping = {
    # Identically named
    'libdmx':            'libdmx',
    'libfontenc':        'libfontenc',
    'liblbxutil':        'liblbxutil',
    'libpciaccess':      'libpciaccess',
    'libxkbcommon':      'libxkbcommon',
    'libxkbui':          'libxkbui',
    'libxkbfile':        'libxkbfile',
    'libxtrans':         'libxtrans',
    'wayland':           'wayland',

    # Changed names that don't follow the standard rules
    'wayland-demos':     'wayland',
    'libdrm':            'drm',
    'drm-snapshot':      'drm',
    'xorg-server':       'xserver',
    'x11proto-core':     'x11proto',
    'xfonts-encodings':  'encodings',
    'libfs':             'libFS',
    'libice':            'libICE',
    'libsm':             'libSM',
    'libxcalibrate':     'libXCalibrate',
    'libxres':           'libXRes',
    'libxscrnsaver':     'libXScrnSaver',
    'libxtrap':          'libXTrap',
    'libxprintapputil':  'libXprintAppUtil',
    'libxvmc':           'libXvMC',
    'libxprintutil':     'libXprintUtil',
}
# Note: Duplicate values will get mapped to just one key
fdo_to_deb_mapping = invert_dict(deb_to_fdo_mapping)


# "Standard" Debian-X renaming rules
fdo_to_deb_rename_rules = [
    { 're': r'^xf86-(.*)$', 'sub': r'xserver-xorg-\1', },
    { 're': r'^(.*)proto$', 'sub': r'x11proto-\1', },
]
compile_table(fdo_to_deb_rename_rules)

# Inverse of Debian-X renaming rules
deb_to_fdo_rename_rules = [
    { 're': r'^lib([a-z])',         'sub': r'lib\1',       'func': group_match_capitalize},
    { 're': r'^lib(.*)wm$',         'sub': r'lib\1WM',     },
    { 're': r'^xtrans(.*)',         'sub': r'libxtrans\1', },
    { 're': r'^x11proto-(.*)$',     'sub': r'\1proto',     },
    { 're': r'^xserver-xorg-(.*)$', 'sub': r'xf86-\1',     },
]
compile_table(deb_to_fdo_rename_rules)

def lookup(name, mapping, rules):
    if name is None:
        return None

    # Lookup the package name
    pkg = mapping.get(name, None)
    if pkg is not None:
        return pkg

    # Use standard rename rules
    for rule in rules:
        m = rule['rc'].search(name)
        if not m:
            continue
        if 'func' in rule:
            text = re.sub(rule['re'], rule['func'], name)
            pat = str(rule.get('sub',None))
            return str.replace(pat, '\\1', text)
        else:
            return rule['rc'].sub(rule['sub'], name)

    # Not found; assume the same package name applies
    return name

def debpkg_to_fdopkg(name):
    return lookup(name, deb_to_fdo_mapping, deb_to_fdo_rename_rules)

def fdopkg_to_debpkg(name):
    return lookup(name, fdo_to_deb_mapping, fdo_to_deb_rename_rules)

if __name__ == "__main__":
    import sys

    if len(sys.argv) < 2:
        print("Usage: %prog <package-name>")
        sys.exit(1)

    package = sys.argv[1]

    print(lookup(package,
                 deb_to_fdo_mapping,
                 deb_to_fdo_rename_rules))



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import urllib2

def readurl(url):
    try:
        fin = urllib2.urlopen(url)
        content = fin.read()
        fin.close()
        return content
    except:
        return None



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2010-2012 Bryce Harrington <bryce@canonical.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import, print_function, unicode_literals

import gtk

class Application(object):
    def __init__(self):
        self.pages = {}
        self.current_page = None

        # Top level UI containers
        self.window = self.create_top_window()
        self.frame = self.create_top_frame()

        # Create UI elements
        self.accelerators = gtk.AccelGroup()

        # Assemble the UI
        self.window.add(self.frame)

        # Positions
        self.window.set_default_size(400, 300)
        self.window.set_size_request(400, 300)

    def create_top_window(self):
        win = gtk.Window(gtk.WINDOW_TOPLEVEL)
        win.connect('delete_event', self.on_delete)
        win.set_border_width(10)
        return win

    def create_top_frame(self):
        frame = gtk.Frame("")
        self.page_title = gtk.Label("")
        frame.set_label_widget(self.page_title)
        frame.set_label_align(0.0, 0.0)
        return frame

    def create_button(self, primary_text, secondary_text):
        button = gtk.Button(primary_text)
        button_label = button.get_children()[0]
        button_label.set_markup(
            "<b>%s</b>\n<small>%s</small>"
            %(primary_text, secondary_text))
        button_label.set_alignment(0.0, 0.0)
        button.set_relief(gtk.RELIEF_NONE)
        button.show()
        return button

    def create_nav_bar(self, next_page=None, prev_page=None):
        hbox_navigation = gtk.HButtonBox()

        if prev_page is not None:
            hbox_navigation.set_property("layout-style", gtk.BUTTONBOX_START)
            prev_button = gtk.Button("Back")
            prev_button.connect("clicked", self.on_page, prev_page)
            hbox_navigation.pack_start(prev_button)
        if next_page is not None:
            hbox_navigation.set_property("layout-style", gtk.BUTTONBOX_END)
            next_button = gtk.Button("Next")
            next_button.connect("clicked", self.on_page, next_page)
            hbox_navigation.pack_end(next_button)
        return hbox_navigation

    def create_page(self, text):
        page = gtk.VBox(spacing=5)
        page.set_border_width(10)
        label = gtk.Label(text)
        label.set_line_wrap(True)
        label.set_use_markup(True)
        label.set_alignment(0.0, 0.0)
        page.pack_start(label, expand=False)
        return page

    def on_delete(self, widget, event):
        gtk.main_quit()

    def on_close(self, widget):
        gtk.main_quit()



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2010-2012 Bryce Harrington <bryce@canonical.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import, print_function, unicode_literals

import os
from gi.repository import Gtk
import shutil

import gettext
from gettext import gettext as _
gettext.textdomain('xdiagnose')

from .application import Application
from .utils.debug import dbg
from .utils.processes import execute


class XDiagnoseApplication(Application):
    def __init__(self):
        Application.__init__(self)
        self.xorg_conf_path = "/etc/X11/xorg.conf"
        self.xorg_conf_d_path = "/etc/X11/xorg.conf.d"
        self.xorg_conf_backup_path = "%s-backup" %(self.xorg_conf_path)
        self.xorg_conf_d_backup_path = "%s-backup" %(self.xorg_conf_d_path)

        self.pages['welcome'] = self.create_welcome_page()
        self.pages['actions'] = self.create_actions_page()
        self.pages['reconfigure'] = self.create_reconfigure_page()
        self.pages['troubleshoot'] = self.create_troubleshoot_page()

        # Refresh
        self.update_frame()
        self.window.show_all()
        self.on_page(None, 'welcome')

    def create_welcome_page(self):
        page = self.create_page(
            '<big>The system is running in low-graphics mode</big>\n\n' +
            'Your graphics and input devices could not be detected correctly.  You will need to configure these yourself.')
        nav_bar = self.create_nav_bar(next_page="actions")
        page.pack_end(nav_bar, expand=False)
        return page

    def create_actions_page(self):
        page = self.create_page("What would you like to try?")

        hbox = Gtk.HBox()

        button1 = self.create_button("Run in low-graphics mode",
                                      "Use low-graphics for just one session")
        button1.connect("clicked", self.on_low_res_mode)
        button2 = self.create_button("Reconfigure",
                                      "Try to automatically fix the problem")
        button2.connect("clicked", self.on_page, "reconfigure")
        button3 = self.create_button("Troubleshoot",
                                      "Review errors, configs, and logs")
        button3.connect("clicked", self.on_page, "troubleshoot")
        button4 = self.create_button("Console",
                                      "Exit to the console login")
        button4.connect("clicked", self.on_exit_to_console)

        table = Gtk.Table(rows=4, columns=3, homogeneous=False)
        table.set_border_width(0)
        table.attach(Gtk.Label(''), 0, 1, 0, 1, xoptions=Gtk.EXPAND)
        table.attach(button1, 1, 2, 0, 1)
        table.attach(button2, 1, 2, 1, 2)
        table.attach(button3, 1, 2, 2, 3)
        table.attach(button4, 1, 2, 3, 4)
        table.attach(Gtk.Label(''), 2, 3, 0, 1, xoptions=Gtk.EXPAND)
        page.pack_start(table, expand=False)

        nav_bar = self.create_nav_bar(prev_page="welcome")
        page.pack_end(nav_bar, expand=False)
        return page

    def create_reconfigure_page(self):
        page = self.create_page("How would you like to reconfigure your display?")

        hbox = Gtk.HBox()

        button1 = self.create_button("Reset to defaults",
                                      "Use default (generic) configuration")
        button1.connect("clicked", self.on_reset_to_default_xorg_conf)
        button2 = self.create_button("Generate xorg.conf",
                                      "Creates a new configuration for this hardware")
        button2.connect("clicked", self.on_generate_xorg_conf)
        button3 = self.create_button("Restore backup",
                                      "Use your backed-up configuration")
        button3.connect("clicked", self.on_restore_backup)

        table = Gtk.Table(rows=3, columns=3, homogeneous=False)
        table.set_border_width(0)
        table.attach(Gtk.Label(''), 0, 1, 0, 1, xoptions=Gtk.EXPAND)
        table.attach(button1, 1, 2, 0, 1)
        table.attach(button2, 1, 2, 1, 2)
        table.attach(button3, 1, 2, 2, 3)
        table.attach(Gtk.Label(''), 2, 3, 0, 1, xoptions=Gtk.EXPAND)
        page.pack_start(table, expand=False)

        nav_bar = self.create_nav_bar(prev_page="actions")
        page.pack_end(nav_bar, expand=False)
        return page

    def create_troubleshoot_page(self):
        page = self.create_page("What information would you like to review?")

        hbox = Gtk.HBox()

        button1 = self.create_button("X server log",
                                      "Review the failed session Xorg.0.log")
        button1.connect("clicked", self.on_view_file, "/var/log/Xorg.0.log")
        button2 = self.create_button("lightdm's X startup log",
                                      "Review the display manager log")
        button2.connect("clicked", self.on_view_file, "/var/log/lightdm/x-0.log")
        button3 = self.create_button("Edit X Config",
                                      "Edit xorg.conf configuration file")
        button3.connect("clicked", self.on_edit_file, "/etc/X11/xorg.conf")

        table = Gtk.Table(rows=4, columns=3, homogeneous=False)
        table.set_border_width(0)
        table.attach(Gtk.Label(''), 0, 1, 0, 1, xoptions=Gtk.EXPAND)
        table.attach(button1, 1, 2, 0, 1)
        table.attach(button2, 1, 2, 1, 2)
        table.attach(button3, 1, 2, 2, 3)
        table.attach(button4, 1, 2, 3, 4)
        table.attach(Gtk.Label(''), 2, 3, 0, 1, xoptions=Gtk.EXPAND)
        page.pack_start(table, expand=False)

        nav_bar = self.create_nav_bar(prev_page="actions")
        page.pack_end(nav_bar, expand=False)
        return page

    def has_backup(self):
        if (os.path.exists(self.xorg_conf_backup_path) or
            os.path.exists(self.xorg_conf_d_backup_path)):
            return True
        return False

    def backup_xorg_conf(self):
        '''Backs up xorg.conf and xorg.conf.d, overwriting any pre-existing backups'''
        if os.path.exists(self.xorg_conf_path):
            if os.path.exists(self.xorg_conf_backup_path):
                shutil.rm(self.xorg_conf_backup_path)
            shutil.copy(self.xorg_conf_path, self.xorg_conf_backup_path)
        if os.path.exists(self.xorg_conf_d_path):
            if os.path.exists(self.xorg_conf_d_backup_path):
                shutil.rmtree(self.xorg_conf_d_backup_path)
            shutil.copytree(self.xorg_conf_d_path, self.xorg_conf_d_backup_path)

    def update_frame(self):
        self.window.ensure_style()
        style = self.window.get_style()
        color = style.bg[Gtk.STATE_SELECTED]
        self.frame.modify_bg(Gtk.STATE_NORMAL, color)

        self.page_title.modify_fg(Gtk.STATE_NORMAL, color)

    def on_page(self, widget, page_name):
        new_page = self.pages[page_name]
        if self.current_page is not None:
            self.frame.remove(self.current_page)
        if page_name == "welcome":
            self.page_title.set_markup("<big>Failsafe-X</big>")
        else:
            self.page_title.set_markup("<big>Failsafe-X: %s</big>" %(page_name.capitalize()))
        self.frame.add(new_page)
        self.current_page = new_page
        self.frame.show_all()

    def on_low_res_mode(self, widget):
        pass

    def on_exit_to_console(self, widget):
        pass

    def on_reset_to_default_xorg_conf(self, widget):
        if not self.backup_xorg_conf():
            dbg("Error: Could not backup the config")
            return
        shutils.rm(self.xorg_conf)
        shutils.rmtree(self.xorg_conf_d)

    def on_generate_xorg_conf(self, widget):
        if not self.backup_xorg_conf():
            dbg("Error: Could not backup the config")
            return
        pass

    def on_restore_backup(self, widget):
        pass

    def on_view_file(self, widget, filename):
        execute(["zenity",
                 "--text-info",
                 "--filename=%s" %(filename),
                 "--width=640",
                 "--height=480"])

    def on_edit_file(self, widget, filename):
        if not os.path.exists(self.xorg_conf_path):
            # TODO: Stub in an xorg.conf
            pass
        execute(["zenity",
                 "--text-info",
                 "--filename=%s" %(self.xorg_conf_path),
                 "--width=640",
                 "--height=480"])


if __name__ == "__main__":
    import sys

    dbg("main:  Starting program")
    app = XDiagnoseApplication()
    Gtk.main()

    dbg("main:  Ending program")
    sys.exit(0)





#!/usr/bin/python3
# -*- coding: utf-8 -*-

import urllib
import lxml.html

from text import o2str

def tables_from_url(url):
    data = urllib.urlopen(url).read().decode('utf-8', 'replace')
    tree = lxml.html.fromstring(o2str(data))

    tables = []
    for tbl in tree.iterfind('.//table'):
        tele = []
        for tr in tbl.iterfind('.//tr'):
            try:
                text = [e.strip() for e in tr.xpath('.//text()') if
                len(e.strip()) > 0]
                tele.append(text)
            except:
                print(tr)
                raise
        yield tele

def data_from_url(url):
    '''Looks up first non-trivial data table from url'''
    for t in tables_from_url(url):
        if len(t) >= 5:
            return t

    raise Exception("No usable data returned from %s" %(url))


# vi:set ts=4 sw=4 expandtab:



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2010-2012 Bryce Harrington <bryce@canonical.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


'''
This updates lines in config files matching the following syntax.  It
preserves the rest of the file contents unchanged.

PARAMETER_FIRST="some string values"
PARAMETER_SECOND=""

# This is a comment
PARAMETER_THIRD="0x01234567,0xfefefefe,0x89abcdef,0xefefefef"
PARAMETER_FOURTH=false
PARAMETER_FIFTH=42
'''

from __future__ import absolute_import, print_function, unicode_literals

import re
import os
import sys
import shutil
import fileinput

from .utils.debug import dbg

def rotated_backup(path, count=4):
    """
    Rename a file keeping up to <count> prior versions
    """
    # Rotate the old backups
    for i in range(count, 1, -1):
        old_path = "%s.bak.%d" %(path, i-1)
        new_path = "%s.bak.%d" %(path, i)
        if os.path.exists(old_path):
            shutil.move(old_path, new_path)

    new_path = old_path
    old_path = "%s.bak" %(path)
    if os.path.exists(old_path):
        shutil.move(old_path, new_path)

    new_path = old_path
    if os.path.isfile(path):
        shutil.copy(path, new_path)
    elif os.path.isdir(path):
        shutil.copytree(path, new_path)
    else:
        shutil.move(path, new_path)


def safe_backup(path, keep_original=True):
    """
    Rename a file or directory safely without overwriting an existing
    backup of the same name.
    """
    count = -1
    new_path = None
    while True:
        if os.path.exists(path):
            if count == -1:
                new_path = "%s.bak" % (path)
            else:
                new_path = "%s.bak.%s" % (path, count)
            if os.path.exists(new_path):
                count += 1
                continue
            else:
                if keep_original:
                    if os.path.islink(path):
                        linkto = os.readlink(path)
                        shutil.copy(linkto, new_path)
                    elif os.path.isfile(path):
                        shutil.copy(path, new_path)
                    elif os.path.isdir(path):
                        shutil.copytree(path, new_path)
                    else:
                        shutil.move(path, new_path)
                    break
        else:
            break
    return new_path

def config_dict(filename, delim='='):
    re_param = re.compile("^\s*(\w+)\s*"+delim+"\s*(.*)")
    data = {}
    for line in fileinput.input(filename):
        m = re_param.match(line)
        if m:
            data[m.group(1)] = m.group(2)
    return data

# TODO: Perhaps the filename should be a fileio too?
def config_update(filename, override_params=None, merge_params=None, delim='=', fileio=sys.stdout):
    '''filename is the input file.  fileio is the output stream'''
    keys = []
    if override_params:
        keys = list(override_params.keys())
    if merge_params:
        keys.extend(list(merge_params.keys()))
    keys = list(set(keys))
    keys.sort()

    dbg(filename)
    for line in fileinput.input(filename):
        dbg(line)
        new_line = line

        if merge_params is not None:
            dbg("Merging parameters")
            for key in merge_params:
                dbg(" - %s" %(key))
                p = re.compile("^\s*"+key+"\s*"+delim+"\s*(\"?)(.*)(\"?)")
                m = p.match(line)
                if m:
                    value = merge_params[key].replace('"', '')
                    if len(value)>0:
                        new_line = "%s%s%s%s %s%s\n" %(key, delim, m.group(1), m.group(2), value, m.group(3))
                    keys.remove(key)

        if override_params is not None:
            dbg("Overriding parameters")
            for key in list(override_params.keys()):
                dbg(" - %s" %(key))
                p = re.compile("^\s*"+key+"\s*"+delim)
                if p.match(line):
                    new_line = "%s%s%s\n" %(key, delim, override_params[key])
                    dbg("   delim: %s" %(delim))
                    dbg("   param: %s" %(override_params[key]))
                    if key in keys:
                        keys.remove(key)

        fileio.write(new_line)
        dbg("Wrote: %s" %(new_line))

    # Handle case of parameters that weren't already present in the file
    for key in keys:
        dbg("Adding key %s" %(key))
        if override_params and key in override_params:
            fileio.write("%s%s%s\n" %(key, delim, override_params[key]))
        elif merge_params and key in merge_params:
            fileio.write("%s%s%s\n" %(key, delim, merge_params[key]))

if __name__ == '__main__':
    filename = '/etc/default/grub'
    override_params = {
        'FOO':                        '"xyz"',
        'BOTH':                       '"correct"',
        'GRUB_DEFAULT':               2,
        'GRUB_CMDLINE_LINUX':         '"foo=bar"',
        'GRUB_HIDDEN_TIMEOUT_QUIET':  False,
        }
    merge_params = {
        'GRUB_CMDLINE_LINUX_DEFAULT': '"vesafb.invalid=1"',
        'BAR':                        'f(1&&2*$i^2) # \o/',
        'BOTH':                        '"incorrect"',
        }

    config_update(filename, override_params, None)
    config_update(filename, None,            merge_params)
    config_update(filename, override_params, merge_params)

    # TODO: Test for if drm.debug=0x4 and we want to set it to 0xe



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2011-2012 Bryce Harrington <bryce@bryceharrington.org>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import, print_function, unicode_literals

'''Command line options'''

import os.path
from optparse import OptionParser

class OptionHandler(OptionParser):
    '''Subclass of OptionParser that also tracks descriptions'''
    def __init__(self, info, app_name=None, arg_names=''):
        '''Creates an OptionParser instance for the options in this module'''
        prog = info.PROGNAME
        usage = None
        if app_name:
            app_name = os.path.basename(app_name)
            prog = "%s %s" %(app_name, info.PROGNAME)
            usage = "%s %s" %(app_name, arg_names)
        version = info.VERSION or "(UNRELEASED)"
        OptionParser.__init__(
            self,
            usage=usage,
            version="%s %s" %(prog, version),
            epilog="%s - %s" %(info.PROGNAME, info.SHORT_DESCRIPTION)
            )
        self.descriptions = []

    def add(self, short_opt, long_opt, **kwargs):
        '''Adds an option.

        Example::

          opt_hand.add("-d", "--debug",
                       help="Enable debug output",
                       action="store_true", default=False, dest="debug",
                       desc="Turns on verbose debugging output")
        '''
        item = {
            'opts': [short_opt, long_opt],
            'text': kwargs.get('desc', ''),
            }
        self.descriptions.append(item)
        del kwargs['desc']
        self.add_option(short_opt, long_opt, **kwargs)




#!/usr/bin/python3

import os

from file_io import load_file
from lists import to_list
from text import to_bool

class InvalidConfig():
    def __init__(self, item):
        self.item = item
    def __str__(self):
        return "Invalid configuration: %s" %(self.item)

class Config(object):
    '''Yet another INI-style config file parser.

    Assumes a "key = value" style file, with # anywhere on line to
    indicate a comment.

    Lines can be continued with either backslash (\) or a trailing
    comma, if the subsequent line is space-indented.

    All keys from the file are loaded as data members of this class so
    can be easily referenced as "config.key".

    If a key name includes one or more periods (.) it is converted into
    a dict.  So, "foo.bar.baz = doh.blah, 42.1" in the config file would be
    referenced in code as "foo['bar']['baz'] = 'doh.blah, 42.1'.

    The 'include' keyword is supported as a way to import the contents
    of another file, which is then parsed and handled as above, with all
    elements brought into the current namespace.
    '''
    def __init__(self, filename=None, lines=None):
        if filename is not None:
            self._filename = os.path.expanduser(filename)
            if os.path.exists(self._filename):
                self.load(load_file(self._filename))
        if lines is not None:
            self.load(lines=lines)

    @property
    def filename(self):
        """The name of the file the config was loaded from.

        Returns None if config was provided directly during
        initialization.
        """
        try:
            return self._filename
        except:
            return None

    def clear(self):
        """Deletes all config data from object"""
        for key, value in self.__dict__.items():
            if key.startswith('_'):
                continue
            del self.__dict__[key]

    def copy(self, obj):
        """Copies contents of another config object"""
        if obj is None or len(obj.__dict__.keys()) < 1:
            return

        for key, value in obj.__dict__.items():
            if key.startswith('_'):
                continue
            old_value = self.__dict__.get(key, None)
            if old_value is not None and type(old_value) != type(value):
                if type(old_value) is list and type(value) is str:
                    value = to_list(value)
                else:
                    raise InvalidConfig("key %s (type %s) given for a type %s" %(
                            key, type(value), type(old_value)))
            self.__dict__[key] = value

    def set(self, option, value):
        """Sets an option, handling dots as a path of dicts"""
        def _recurse_set(parent_dict, fields, value):
            field = fields[0]
            if len(fields) == 1:
                if type(parent_dict.get(field,None)) is list:
                    parent_dict[field] = to_list(value)
                elif type(parent_dict.get(field,None)) is bool:
                    parent_dict[field] = to_bool(value)
                else:
                    parent_dict[field] = value
                return
            if field not in parent_dict:
                parent_dict[field] = {}
            elif type(parent_dict[field]) is not dict:
                buf = parent_dict[field]
                parent_dict[field] = {'': buf}
            _recurse_set(parent_dict[field], fields[1:], value)

        _recurse_set(self.__dict__, option.split('.'), value)

    def get(self, option, default=None):
        """Retrieves an option, with dots navigating dict tree"""
        def _recurse_get(parent_dict, fields):
            field = fields[0]
            if field not in parent_dict:
                parent_dict[field] = ''
            if type(parent_dict[field]) is not dict:
                return parent_dict[field]
            return _recurse_get(parent_dict[field], fields[1:])

        return _recurse_get(self.__dict__, option.split('.'))

    def load(self, lines=None):
        """Parses given lines into the config"""
        if not lines:
            return
        if type(lines) is not list:
            lines = lines.split("\n")

        assert(type(lines) is list)
        possible_continuation = False
        last_option = None
        for line in lines:
            if len(line.strip()) == 0:
                last_option = None
                possible_continuation = False
                continue
            # We can continue only if this line starts with space and
            # the prior line ended in a continuation character (\ or ,)
            if possible_continuation and not line[0].isspace():
                possible_continuation = False
                last_option = None
            line = line.split('#')[0].strip()

            # Check for possible continuation to the next line
            if line.endswith('\\'):
                possible_continuation = True
                line = line[:-1].strip()
            elif line.endswith(','):
                possible_continuation = True

            # Add the option to ourself
            if '=' in line:
                option, value = line.split('=', 1)
                option = option.strip()
                if option:
                    last_option = option
                    self.set(option, value.strip())

            # Line continues from previous, just append to prior value
            elif possible_continuation and last_option:
                old_value = self.get(last_option)
                if type(old_value) is list:
                    old_value.extend(to_list(line))
                else:
                    old_value += " " + line
                self.set(last_option, old_value)

            # Import another config file
            elif line[:8] == 'include ':
                filename = line[8:].strip()
                lines = load_file(filename)
                self.load(lines)
                possible_continuation = False
                last_option = None

    def __str__(self):
        def _value_to_str(key, value):
            assert(type(value) is not dict)
            if value is None:
                return "%s=\n" %(key)
            elif type(value) is list:
                return "%s=%s\n" %(key, ', '.join(str(x) for x in value))
            else:
                return "%s=%s\n" %(key, str(value))
        def _items_to_str(parent, items):
            text = ''
            for key, value in items:
                if parent is not None:
                    param = "%s.%s" %(parent, key)
                else:
                    param = key
                if type(value) is dict:
                    text += _items_to_str(param,value.items())
                else:
                    text += _value_to_str(param,value)
            return text

        return _items_to_str(None, self.__iter__())

    def __len__(self):
        return len(list(self.__iter__()))

    def __getitem__(self, key):
        return self.__dict__.get(key, None)

    def __iter__(self):
        keys = self.__dict__.items()
        keys.sort()
        for key,value in keys:
            if key.startswith('_') or key=='':
                continue
            yield (key,value)

    def __contains__(self, item):
        if item.startswith('_'):
            return False
        return item in self.__dict__.keys()

if __name__ == "__main__":
    #config = Config("~/.config/user-dirs.dirs")
    #config = Config("~/.taskhelmrc")
    config = Config("~/.taskrc")
    print(config)
    print()
    print(config.shell['prompt'])
    print(config.color)

    config.clear()
    assert( str(config) == '')



#!/usr/bin/python3

# TODO: Switch to a dict data structure for children
#       See logic in config.py as example.
#       This will give faster lookups using less code.
#       But loses ordering...
#       Ordering could be handled by separate list of keys

class Node(object):
    '''Wraps an object for storing into a tree structure.

    For identification purposes, value needs to either be a
    simple string, a dict with an id field, or an instance
    of a class that has an id attribute.

    The value item must also be convertable to str.
    '''
    def __init__(self, value):
        self.value = value
        self.children = []

    def __str__(self):
        """
        Return a text representation of the node and its children,
        shown in a bulleted outline format.
        """
        text = ''
        for depth, item in self.flatten():
            if depth:
                text += "%s+ %s\n" %('  '*depth, item)
            else:
                text += "%s\n" %(item)
        return text

    def add_child(parent_node, fields):
        """Append a descendants to the parent's child list.

        fields is a lineage of child, grandchild, etc.
        """
        assert(parent_node)
        if len(fields)<=0:
            return

        field = fields[0]
        node = None
        for child in parent_node.children:
            assert(child)
            if child.value == field:
                node = child
        if node is None:
            node = Node(field)
            parent_node.children.append(node)
        if len(fields) > 1:
            node.add_child(fields[1:])

    def find(self, key):
        """
        Locate the item with the value matching the given key.
        If multiple items have the same key, returns the first
        one found.
        """
        if self.value is not None:
            if type(self.value) == type(key):
                if self.value == key:
                    return self
            elif type(self.value) is dict:
                id = self.value.get('id', None)
                if id == key:
                    return self
            elif hasattr(obj, 'id'):
                id = self.value.id
                if id == key:
                    return self

        for node in self.children:
            result = node.find(key)
            if result is not None:
                return result
        return None

    def flatten(self, current_level=0):
        """
        Creates a list representation of the tree.  Each
        element of the list is a tuple of the depth and value.
        """
        items = []
        if self.value:
            items.append((current_level, self.value))
        for node in self.children:
            child_items = node.flatten(current_level+1)
            items.extend(child_items)
        return items


def list_to_tree(items):
    root = Node('root')
    for item in items:
        root.add_child(item.split('.'))
    return root


if __name__ == "__main__":

    def test_create_node():
        node = Node("parent")
        node.add_child("child")
        assert(1 == len(node.children))

    def test_add_child():
        node = Node("parent")
        node.add_child("child")
        assert(1 == len(node.children))

    def test_add_child():
        node = Node("parent")
        node.add_child("a")
        node.add_child("b")
        node.add_child( "c")
        assert(3 == len(node.children))

    def test_create_tree():
        node = Node("parent")
        node.add_child("a")
        node.children[0].add_child("1")
        node.children[0].add_child("2")
        node.add_child("b")
        node.children[1].add_child("1")
        node.children[1].add_child("2")
        assert(2 == len(node.children))

    def test_children_one_level():
        data = [
            'a',
            'b',
            'c',
            ]
        repr = """root
  + a
  + b
  + c
"""
        tree = list_to_tree(data)
        assert(repr == str(tree))

    def test_children_two_levels():
        data = [
            'a.1',
            'a.2',
            'a.3',
            ]
        repr = """root
  + a
    + 1
    + 2
    + 3
"""
        tree = list_to_tree(data)
        assert(repr == str(tree))

    def test_children_three_levels():
        data = [
            'a.1.1',
            'a.2.1',
            'a.3.1',
            ]
        repr = """root
  + a
    + 1
      + 1
    + 2
      + 1
    + 3
      + 1
"""
        tree = list_to_tree(data)
        assert(repr == str(tree))

    def test_children_tree():
        data = [
            'a.1.1',
            'a.1.2',
            'a.2.1',
            ]
        repr = """root
  + a
    + 1
      + 1
      + 2
    + 2
      + 1
"""
        tree = list_to_tree(data)
        assert(repr == str(tree))

    def test_flatten_empty():
        data = []
        tree = list_to_tree(data)
        items = tree.flatten()
        assert((0,'root') == items[0])

    def test_flatten_single():
        data = ['a']
        tree = list_to_tree(data)
        items = tree.flatten()
        assert((1,'a') == items[1])

    def test_flatten_set():
        data = ['a', 'b', 'c']
        tree = list_to_tree(data)
        items = tree.flatten()
        assert(4 == len(items))

    def test_flatten_deep():
        data = ['a.b.c.d.e']
        tree = list_to_tree(data)
        items = tree.flatten()
        assert(6 == len(items))

    def test_flatten_tree():
        data = ['a.1.1',
                'a.1.2',
                'a.2',
                'a.2.1',
                'b.1',
                'c.1.1',
                ]
        tree = list_to_tree(data)
        items = tree.flatten()
        assert(12 == len(items))

    def test_find():
        data = ['a']
        tree = list_to_tree(data)
        node = tree.find('a')
        assert('a' == node.value)

    def test_find_fail():
        data = ['a']
        tree = list_to_tree(data)
        node = tree.find('b')
        assert(None == node)

    def test_find_deep():
        data = ['a.b.c.d.e.f']
        tree = list_to_tree(data)
        node = tree.find('e')
        assert('e' == node.value)

    def test_find_in_set():
        data = ['a.1',
                'a.2',
                'a.3',
                'b.4',
                'b.5',
                'c.6'
                ]
        tree = list_to_tree(data)
        node = tree.find('5')
        assert('5' == node.value)

    def test_find_from_multiple():
        data = ['a.1',
                'a.2',
                'a.3',
                'b.1',
                'b.2',
                'c.1'
                ]
        tree = list_to_tree(data)
        node = tree.find('2')
        assert('2' == node.value)

    test_create_node()
    test_add_child()
    test_add_child()
    test_create_tree()
    test_children_one_level()
    test_children_two_levels()
    test_children_three_levels()
    test_children_tree()
    test_flatten_empty()
    test_flatten_single()
    test_flatten_set()
    test_flatten_deep()
    test_flatten_tree()
    test_find()
    test_find_fail()
    test_find_deep()
    test_find_in_set()
    test_find_from_multiple()

    data = [
        'a.a.a',
        'a.b.a',
        'a.b.b',
        'a.b.c',
        'b.a',
        'c',
        'd',
        'd.a.a',
        ]

    tree = list_to_tree(data)
    print(tree)

    # TODO: Convert back to dotted list



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2010-2012 Bryce Harrington <bryce@canonical.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import os
import re
import glob
import DistUtilsExtra.auto

from xdiagnose import info

changelog = "debian/changelog"
if os.path.exists(changelog):
    head=open(changelog).readline()
    match = re.compile(".*\((.*)\).*").match(head)
    if match:
        version = match.group(1)
# TODO: Also set the VERSION in info.py

DistUtilsExtra.auto.setup(
    name             = info.PROGNAME,
    version          = version,
    url              = info.URL,
    author           = info.LEAD_DEVELOPER.name,
    author_email     = info.LEAD_DEVELOPER.email,
    description      = info.SHORT_DESCRIPTION,
    long_description = info.DESCRIPTION,
    license          = info.LICENSE_URL,
    platforms        = ['any'],
    requires         = ['gobject', 'pygtk'],
    # Note: Requires pep8 and pydoctor for make lint
    packages     = [
        'xdiagnose',
        'xdiagnose/utils',
        ],
    package_data = {
        },
    data_files   = [
        ('share/xdiagnose/icons',         ['data/icons/microscope.svg']),
        ('share/xdiagnose/workloads',     glob.glob('data/workloads/*')),
        ('share/xdiagnose',               glob.glob('data/failsafe*')),
        ('share/xdiagnose',               ['data/bubu.lib']),
        ],
    scripts      = [
        'bin/dpkg-log-summary',
        'bin/xdiagnose',
        'bin/xedid',
        'bin/xrandr-tool',
        'bin/xrotate',
        'bin/xpci',
        ],
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

def dicts_equal(dict_a, dict_b):
    if dict_a is None or dict_b is None:
        return False

    # Check if keys are the same
    if set(dict_a.keys()) != set(dict_b.keys()):
        return False

    # Check if values are the same
    for a,b in zip(dict_a.iteritems(), dict_b.iteritems()):
        if a != b:
            return False

    return True

# vi:set ts=4 sw=4 expandtab:



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from os.path import exists

class FileDoesntExist(BaseException):
    def __init__(self, file_name=None):
        self.file_name = file_name
    def __str__(self):
        return "The file (%s) does not exist" %(self.file_name)

def load_binary(filename):
    assert(filename)
    if not exists(filename):
        raise FileDoesntExist(filename)
    with open(filename, 'rb') as file:
        data = file.read()
    return data

def load_file(filename):
    assert(filename)
    if not exists(filename):
        raise FileDoesntExist(filename)
    with open(filename, 'r') as file:
        lines = file.readlines()
    return lines

def write_file(filename, text):
    f = open(filename, "w")
    f.write(text.encode('utf-8'))
    f.close()

# vi:set ts=4 sw=4 expandtab:



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2011-2012 Bryce Harrington <bryce@bryceharrington.org>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import, print_function, unicode_literals

from datetime import (
    datetime,
    timedelta
    )


def week_start(dt=None):
    """
    Calculates the ISO week start datetime for the given time
    @param dt: Datetime to calculate from (defaults to now)
    @returns: datetime of week start
    """
    if dt is None:
        dt = datetime.today()
    week_days = datetime.isoweekday(dt) % 7
    week_start = dt - timedelta(days=week_days)
    return week_start.replace(hour=0, minute=0, second=0, microsecond=0)

def utc_date_to_local_datetime(dt):
    if dt is None:
        return None
    utc_offset = datetime.utcnow() - datetime.now()
    return datetime.strptime(dt, "%Y%m%dT%H%M%SZ") - utc_offset

def utc_date_to_isocalendar(timestamp):
    """Converts '20120216T025651Z' into (year, week, weekday) tuple"""
    if timestamp is None:
        return None
    elif type(timestamp) is tuple:
        # Assume already in tuple form, just pass-thru
        return timestamp
    dt = utc_date_to_local_datetime(timestamp)
    if dt is None:
        return None
    return dt.isocalendar()

def total_seconds(td):
    tsec = (td.microseconds + (td.seconds + td.days * 24 * 3600) * 10**6)/10**6
    return tsec

if __name__ == '__main__':
    print(week_start())

    td = timedelta(days=10, seconds=2.999)
    dt = datetime.today() - td
    print(week_start(dt))

    print("%d == %d\n" %(total_seconds(td), td.total_seconds()))
